# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the Cloud NetApp Files Storage Pools commands."""


import argparse
from typing import Any

from googlecloudsdk.api_lib.netapp import util as netapp_api_util
from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.netapp import flags
from googlecloudsdk.command_lib.netapp import util as netapp_util
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.concepts import concept_parsers


STORAGE_POOLS_LIST_FORMAT = """\
    table(
        name.basename():label=STORAGE_POOL_NAME:sort=1,
        name.segment(3):label=LOCATION,
        serviceLevel,
        capacityGib,
        network,
        state,
        volumeCapacityGib,
        type
    )"""


## Helper functions to add args / flags for Storage Pools gcloud commands ##


def GetStoragePoolServiceLevelArg(messages, required=True):
  """Adds a --service-level arg to the given parser.

  Args:
    messages: The messages module.
    required: bool, whether choice arg is required or not

  Returns:
    the choice arg.
  """
  custom_mappings = {
      'PREMIUM': (
          'premium',
          """
                          Premium Service Level for Cloud NetApp Storage Pool.
                          The Premium Service Level has a throughput per GiB of
                          allocated volume size of 64 KiB/s.""",
      ),
      'EXTREME': (
          'extreme',
          """
                          Extreme Service Level for Cloud NetApp Storage Pool.
                          The Extreme Service Level has a throughput per GiB of
                          allocated volume size of 128 KiB/s.""",
      ),
      'STANDARD': (
          'standard',
          """
                          Standard Service Level for Cloud NetApp Storage Pool.
                          The Standard Service Level has a throughput per GiB of
                          allocated volume size of 16 KiB/s.""",
      ),
      'FLEX': (
          'flex',
          """
                          Flex Service Level for Cloud NetApp Storage Pool.
                          The Flex Service Level has a throughput per GiB of
                          allocated volume size of 16 KiB/s.""",
      ),
  }
  service_level_arg = arg_utils.ChoiceEnumMapper(
      '--service-level',
      messages.StoragePool.ServiceLevelValueValuesEnum,
      help_str="""The service level for the Cloud NetApp Storage Pool.
       For more details, see:
       https://cloud.google.com/netapp/volumes/docs/configure-and-use/storage-pools/overview#service_levels
        """,
      custom_mappings=custom_mappings,
      required=required,
  )
  return service_level_arg


def GetDirectoryServiceTypeEnumFromArg(choice, messages):
  """Returns the Choice Enum for Directory Service Type.

  Args:
    choice: The choice for directory service type as string
    messages: The messages module.

  Returns:
    the directory service type enum.
  """
  return arg_utils.ChoiceToEnum(
      choice=choice,
      enum_type=messages.ValidateDirectoryServiceRequest.DirectoryServiceTypeValueValuesEnum,
  )


def GetStoragePoolQosTypeArg(messages):
  """Adds the Qos Type arg to the arg parser."""
  qos_type_arg = arg_utils.ChoiceEnumMapper(
      '--qos-type',
      messages.StoragePool.QosTypeValueValuesEnum,
      help_str="""Quality of service (QoS) type for the Storage Pool.""",
  )
  return qos_type_arg


def GetStoragePoolTypeEnumFromArg(choice, messages):
  """Returns the Choice Enum for StoragePoolType."""
  return arg_utils.ChoiceToEnum(
      choice=choice,
      enum_type=messages.StoragePool.TypeValueValuesEnum
  )


def GetStoragePoolScaleTierArg(messages):
  """Returns the ChoiceEnumMapper for StoragePoolScaleTier."""
  return arg_utils.ChoiceEnumMapper(
      '--scale-tier',
      messages.StoragePool.ScaleTierValueValuesEnum,
      help_str=(
          'The scale tier of the Storage Pool. `standard` scale tier'
          ' is for standard capacity and performance tier.'
          ' `enterprise` scale tier is for higher capacity and'
          ' performance tier.'
      ),
      custom_mappings={
          'SCALE_TIER_STANDARD': 'standard',
          'SCALE_TIER_ENTERPRISE': 'enterprise',
      },
      action=actions.DeprecationAction(
          '--scale-tier',
          warn=(
              'The `{flag_name}` flag is deprecated. Use `--scale-type`'
              ' instead.'
          ),
      ),
  )


def GetStoragePoolScaleTierEnumFromArg(choice, messages):
  """Returns the Choice Enum for StoragePoolScaleTier."""
  return GetStoragePoolScaleTierArg(messages).GetEnumForChoice(choice)


def GetStoragePoolScaleTypeArg(messages: Any) -> arg_utils.ChoiceEnumMapper:
  """Returns the ChoiceEnumMapper for StoragePoolScaleType.

  Args:
    messages: The messages module.

  Returns:
    The ChoiceEnumMapper for StoragePoolScaleType.
  """
  return arg_utils.ChoiceEnumMapper(
      '--scale-type',
      messages.StoragePool.ScaleTypeValueValuesEnum,
      help_str=(
          'The scale type of the Storage Pool. `default` scale type'
          ' is for standard capacity and performance. `scaleout` scale'
          ' type is for higher capacity and performance.'
      ),
      custom_mappings={
          'SCALE_TYPE_DEFAULT': 'default',
          'SCALE_TYPE_SCALEOUT': 'scaleout',
      },
  )


def GetStoragePoolScaleTypeEnumFromArg(choice: str, messages: Any) -> Any:
  """Returns the Choice Enum for StoragePoolScaleType.

  Args:
    choice: The choice for storage pool scale type as string.
    messages: The messages module.

  Returns:
    The storage pool scale type enum.
  """
  return GetStoragePoolScaleTypeArg(messages).GetEnumForChoice(choice)


def GetStoragePoolModeEnumFromArg(choice: str, messages: Any):
  """Returns the Choice Enum for StoragePoolMode."""
  return arg_utils.ChoiceToEnum(
      choice=choice, enum_type=messages.StoragePool.ModeValueValuesEnum
  )


def GetStoragePoolModeArg(messages: Any):
  """Returns the ChoiceEnumMapper for StoragePoolMode."""
  return arg_utils.ChoiceEnumMapper(
      '--mode',
      messages.StoragePool.ModeValueValuesEnum,
      help_str="""The mode of the Storage Pool. `ONTAP` mode is for performing ONTAP operations with ONTAP Mode APIs. `DEFAULT` mode is for performing operations allowed with NetApp Files APIs.""",
      custom_mappings={
          'DEFAULT': 'default',
          'ONTAP': 'ontap',
      },
  )


def AddStoragePoolModeArg(parser: argparse.ArgumentParser, messages: Any):
  """Adds the --mode arg to the arg parser for Storage Pools."""
  GetStoragePoolModeArg(messages).choice_arg.AddToParser(parser)


def AddStoragePoolScaleTierArg(parser, messages):
  """Adds the --scale-tier arg to the arg parser for Storage Pools."""
  GetStoragePoolScaleTierArg(messages).choice_arg.AddToParser(parser)


def AddStoragePoolScaleTypeArg(
    parser: argparse.ArgumentParser, messages: Any
) -> None:
  """Adds the --scale-type arg to the arg parser for Storage Pools.

  Args:
    parser: The argparse parser.
    messages: The messages module.
  """
  GetStoragePoolScaleTypeArg(messages).choice_arg.AddToParser(parser)


def AddStoragePoolTypeArg(parser, messages):
  """Adds the --type arg to the arg parser for Storage Pools."""
  type_arg = arg_utils.ChoiceEnumMapper(
      '--type',
      messages.StoragePool.TypeValueValuesEnum,
      help_str='The type of the Storage Pool. `FILE` pools support file-based '
               'volumes only. `UNIFIED` pools support both file and block '
               'volumes.',
      custom_mappings={
          'FILE': ('file', 'File-based volumes only (default).'),
          'UNIFIED': ('unified', 'Both file and block volumes.'),
      })
  type_arg.choice_arg.AddToParser(parser)


def AddStoragePoolServiceLevelArg(
    parser, messages, required=False
):
  GetStoragePoolServiceLevelArg(
      messages, required=required
  ).choice_arg.AddToParser(parser)


def AddStoragePoolAsyncFlag(parser):
  help_text = """Return immediately, without waiting for the operation
  in progress to complete."""
  concepts.ResourceParameterAttributeConfig(name='async', help_text=help_text)
  base.ASYNC_FLAG.AddToParser(parser)


def AddStoragePoolNetworkArg(parser, required=True):
  """Adds a --network arg to the given parser.

  Args:
    parser: argparse parser.
    required: bool whether arg is required or not
  """

  network_arg_spec = {
      'name': str,
      'psa-range': str,
  }

  network_help = """\
        Network configuration for a Cloud NetApp Files Storage Pool. Specifying
        `psa-range` is optional.
        *name*::: The name of the Google Compute Engine
        [VPC network](/compute/docs/networks-and-firewalls#networks) to which
        the volume is connected. Short-form (VPC network ID) or long-form
        (full VPC network name: projects/PROJECT/locations/LOCATION/networks/NETWORK) are both
        accepted, but please use the long-form when attempting to create a Storage Pool using a shared VPC.
        *psa-range*::: This field is not implemented. The values provided in
        this field are ignored.
  """

  parser.add_argument(
      '--network',
      type=arg_parsers.ArgDict(spec=network_arg_spec, required_keys=['name']),
      required=required,
      help=network_help)


def AddStoragePoolActiveDirectoryArg(parser):
  """Adds a --active-directory arg to the given parser."""
  concept_parsers.ConceptParser.ForResource(
      '--active-directory',
      flags.GetActiveDirectoryResourceSpec(),
      'The Active Directory to attach to the Storage Pool.',
      flag_name_overrides={'location': ''}).AddToParser(parser)


def AddStoragePoolKmsConfigArg(parser):
  """Adds a --kms-config arg to the given parser."""
  concept_parsers.ConceptParser.ForResource(
      '--kms-config',
      flags.GetKmsConfigResourceSpec(),
      'The KMS config to attach to the Storage Pool.',
      flag_name_overrides={'location': ''}).AddToParser(parser)


def AddStoragePoolEnableLdapArg(parser):
  """Adds the --enable-ladp arg to the given parser."""
  parser.add_argument(
      '--enable-ldap',
      type=arg_parsers.ArgBoolean(
          truthy_strings=netapp_util.truthy, falsey_strings=netapp_util.falsey),
      help="""Boolean flag indicating whether Storage Pool is a NFS LDAP Storage Pool or not"""
  )


def AddStoragePoolAllowAutoTieringArg(parser):
  """Adds the --allow-auto-tiering arg to the given parser."""
  parser.add_argument(
      '--allow-auto-tiering',
      type=arg_parsers.ArgBoolean(
          truthy_strings=netapp_util.truthy, falsey_strings=netapp_util.falsey),
      help="""Boolean flag indicating whether Storage Pool is allowed to use auto-tiering""",
  )


def AddStoragePoolZoneArg(parser):
  """Adds the Zone arg to the arg parser."""
  parser.add_argument(
      '--zone',
      type=str,
      help="""String indicating active zone of the Storage Pool""",
  )


def AddStoragePoolReplicaZoneArg(parser):
  """Adds the Replica Zone arg to the arg parser."""
  parser.add_argument(
      '--replica-zone',
      type=str,
      help="""String indicating replica zone for the Storage Pool""",
  )


def AddStoragePoolDirectoryServiceTypeArg(parser):
  """Adds the Directory Service Type arg to the arg parser."""
  parser.add_argument(
      '--directory-service-type',
      type=str,
      help="""String indicating directory service type for the Storage Pool""",
  )


def AddStoragePoolCustomPerformanceEnabledArg(parser):
  """Adds the Custom Performance Enabled arg to the arg parser."""
  parser.add_argument(
      '--custom-performance-enabled',
      type=arg_parsers.ArgBoolean(
          truthy_strings=netapp_util.truthy, falsey_strings=netapp_util.falsey
      ),
      help="""Boolean flag indicating whether Storage Pool is a custom performance Storage Pool or not""",
  )


def AddStoragePoolTotalThroughputArg(parser):
  """Adds the Total Throughput arg to the arg parser."""
  parser.add_argument(
      '--total-throughput',
      type=arg_parsers.BinarySize(
          default_unit='MiB/s',
          suggested_binary_size_scales=['MiB/s', 'GiB/s'],
          type_abbr='B/s',
      ),
      help="""The total throughput of the Storage Pool in MiB/s or GiB/s units.
              If no throughput unit is specified, MiB/s is assumed.""",
  )


def AddStoragePoolTotalIopsArg(parser):
  """Adds the Total IOPS arg to the arg parser."""
  parser.add_argument(
      '--total-iops',
      type=int,
      help="""Integer indicating total IOPS of the Storage Pool""",
  )


def AddStoragePoolHotTierSizeArg(parser):
  """Adds the Hot Tier Size arg to the arg parser."""
  parser.add_argument(
      '--hot-tier-size',
      type=arg_parsers.BinarySize(
          default_unit='GiB',
          suggested_binary_size_scales=['GiB'],
          type_abbr='B',
      ),
      help="""The hot tier size of the Storage Pool in GiB units.
              This is a required field when --allow-auto-tiering is set for flex service level.""",
  )


def AddStoragePoolEnableHotTierAutoResizeArg(parser):
  """Adds the Enable Hot Tier Auto Resize arg to the arg parser."""
  parser.add_argument(
      '--enable-hot-tier-auto-resize',
      type=arg_parsers.ArgBoolean(
          truthy_strings=netapp_util.truthy, falsey_strings=netapp_util.falsey
      ),
      help="""Boolean flag indicating whether Storage Pool is allowed to use hot tier auto resize""",
  )


def AddStoragePoolUnifiedPoolArg(parser):
  """Adds the Unified Pool arg to the  parser."""
  parser.add_argument(
      '--unified-pool',
      type=arg_parsers.ArgBoolean(
          truthy_strings=netapp_util.truthy, falsey_strings=netapp_util.falsey
      ),
      help="""Boolean flag indicating whether Storage Pool is a unified pool that supports BLOCK storage. Defaults to False if not specified.""",
      hidden=True,
  )


def AddStoragePoolQosTypeArg(parser, messages):
  GetStoragePoolQosTypeArg(
      messages
  ).choice_arg.AddToParser(parser)

## Helper functions to combine Storage Pools args / flags for gcloud commands ##


def AddStoragePoolCreateArgs(parser, release_track):
  """Add args for creating a Storage Pool."""
  concept_parsers.ConceptParser([
      flags.GetStoragePoolPresentationSpec('The Storage Pool to create.')
  ]).AddToParser(parser)
  flags.AddResourceDescriptionArg(parser, 'Storage Pool')
  flags.AddResourceCapacityArg(parser, 'Storage Pool')
  flags.AddResourceAsyncFlag(parser)
  labels_util.AddCreateLabelsFlags(parser)
  messages = netapp_api_util.GetMessagesModule(release_track=release_track)
  AddStoragePoolServiceLevelArg(
      parser, messages=messages, required=True
  )
  AddStoragePoolNetworkArg(parser)
  AddStoragePoolActiveDirectoryArg(parser)
  AddStoragePoolKmsConfigArg(parser)
  AddStoragePoolEnableLdapArg(parser)
  AddStoragePoolZoneArg(parser)
  AddStoragePoolReplicaZoneArg(parser)
  AddStoragePoolAllowAutoTieringArg(parser)
  AddStoragePoolCustomPerformanceEnabledArg(parser)
  AddStoragePoolTotalThroughputArg(parser)
  AddStoragePoolTotalIopsArg(parser)
  AddStoragePoolTypeArg(parser, messages)
  AddStoragePoolScaleTypeArg(parser, messages)
  AddStoragePoolQosTypeArg(parser, messages)
  AddStoragePoolModeArg(parser, messages)
  if (release_track == base.ReleaseTrack.ALPHA or
      release_track == base.ReleaseTrack.BETA):
    AddStoragePoolHotTierSizeArg(parser)
    AddStoragePoolEnableHotTierAutoResizeArg(parser)
    AddStoragePoolUnifiedPoolArg(parser)
    AddStoragePoolScaleTierArg(parser, messages)


def AddStoragePoolDeleteArgs(parser):
  """Add args for deleting a Storage Pool."""
  concept_parsers.ConceptParser([
      flags.GetStoragePoolPresentationSpec('The Storage Pool to delete.')
  ]).AddToParser(parser)
  flags.AddResourceAsyncFlag(parser)


def AddStoragePoolUpdateArgs(parser, release_track):
  """Add args for updating a Storage Pool."""
  messages = netapp_api_util.GetMessagesModule(release_track=release_track)
  concept_parsers.ConceptParser([
      flags.GetStoragePoolPresentationSpec('The Storage Pool to update.')
  ]).AddToParser(parser)
  flags.AddResourceDescriptionArg(parser, 'Storage Pool')
  flags.AddResourceAsyncFlag(parser)
  flags.AddResourceCapacityArg(parser, 'Storage Pool', required=False)
  labels_util.AddUpdateLabelsFlags(parser)
  AddStoragePoolActiveDirectoryArg(parser)
  AddStoragePoolZoneArg(parser)
  AddStoragePoolReplicaZoneArg(parser)
  AddStoragePoolAllowAutoTieringArg(parser)
  AddStoragePoolTotalThroughputArg(parser)
  AddStoragePoolQosTypeArg(parser, messages)
  AddStoragePoolTotalIopsArg(parser)
  if (release_track == base.ReleaseTrack.ALPHA or
      release_track == base.ReleaseTrack.BETA):
    AddStoragePoolHotTierSizeArg(parser)
    AddStoragePoolEnableHotTierAutoResizeArg(parser)


def AddStoragePoolSwitchArg(parser):
  """Add args for switching zones of a Storage Pool."""
  concept_parsers.ConceptParser([
      flags.GetStoragePoolPresentationSpec('The Storage Pool to switch.')
  ]).AddToParser(parser)
  flags.AddResourceAsyncFlag(parser)


def AddStoragePoolValidateDirectoryServiceArg(parser):
  """Add args for validating directory service of a Storage Pool."""
  concept_parsers.ConceptParser([
      flags.GetStoragePoolPresentationSpec('The Storage Pool to validate.')
  ]).AddToParser(parser)
  flags.AddResourceAsyncFlag(parser)
  AddStoragePoolDirectoryServiceTypeArg(parser)


def AddOntapCommandArg(parser: argparse.ArgumentParser):
  """Adds the ontap command argument to the parser.

  Args:
    parser: The argparse parser.
  """

  parser.add_argument(
      'ontap_command',
      metavar='ONTAP_COMMAND',
      type=str,
      help='The ONTAP command to execute.',
  )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Surface for uploading an SSL certificate to an App Engine app."""


from googlecloudsdk.api_lib.app.api import appengine_ssl_api_client as api_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import flags
from googlecloudsdk.core import log


class Create(base.CreateCommand):
  """Uploads a new SSL certificate.

  The user must be the verified owner of the certificate domain(s). Use the
  gcloud domains command group to manage domain ownership and verification.
  """

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To add a new SSL certificate to App Engine, run:

              $ {command} --display-name='example cert' \
                  --certificate='/home/user/me/my_cert.cer' \
                  --private-key='/home/user/me/my_key.pfx'
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddSslCertificateFlags(parser, required=True)

  def Run(self, args):
    client = api_client.GetApiClientForTrack(self.ReleaseTrack())
    cert = client.CreateSslCertificate(
        args.display_name,
        cert_path=args.certificate,
        private_key_path=args.private_key)
    log.CreatedResource(cert.id)
    return cert



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The register command for BigLake Iceberg REST catalogs tables."""

import textwrap

from googlecloudsdk.api_lib.biglake import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.biglake import arguments
from googlecloudsdk.command_lib.biglake import flags
from googlecloudsdk.core import log


help_text = textwrap.dedent("""\
    To register table `my-table` in parent catalog `my-catalog` and namespace `my-namespace`, run:

      $ {command} my-table --namespace=my-namespace --catalog=my-catalog --metadata-location=gs://my-bucket/metadata.json

    To overwrite table `my-table` in parent catalog `my-catalog` and namespace `my-namespace`, run:

      $ {command} my-table --namespace=my-namespace --catalog=my-catalog --metadata-location=gs://my-bucket/metadata.json --overwrite
    """)


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Register(base.CreateCommand):
  """Register a BigLake Iceberg table."""

  detailed_help = {
      'EXAMPLES': help_text,
  }

  @classmethod
  def Args(cls, parser):
    flags.AddTableResourceArg(parser, 'to register', positional=True)
    arguments.AddTableRegisterArgs(parser)

  def Run(self, args):
    client = util.GetClientInstance(self.ReleaseTrack())
    messages = client.MESSAGES_MODULE

    table_ref = args.CONCEPTS.table.Parse()

    register_request = messages.RegisterIcebergTableRequest(
        name=table_ref.tablesId,
        metadata_location=args.metadata_location,
        overwrite=args.overwrite,
    )

    request = messages.BiglakeIcebergV1RestcatalogV1ProjectsCatalogsNamespacesRegisterRequest(
        parent=util.GetNamespaceName(
            table_ref.catalogsId, table_ref.namespacesId
        ),
        registerIcebergTableRequest=register_request,
    )

    response = (
        client.iceberg_v1_restcatalog_v1_projects_catalogs_namespaces.Register(
            request
        )
    )

    log.CreatedResource(
        util.GetTableName(
            table_ref.catalogsId, table_ref.namespacesId, table_ref.tablesId
        ),
        'table',
    )
    return response



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Remove IAM policy binding from an application."""



from googlecloudsdk.api_lib.apphub import utils as api_lib_utils
from googlecloudsdk.api_lib.apphub.applications import client as apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.apphub import flags
from googlecloudsdk.command_lib.iam import iam_util


_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """\
        To remove an IAM policy binding for the role of `roles/apphub.viewer`
        for the user `test-user@gmail.com` from Application `my-app` in location
        `us-east1`, run:

          $ {command} my-app --location=us-east1 --role=roles/apphub.viewer --member=user:test-user@gmail.com
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class RemoveIamPolicyBindingGA(base.Command):
  """Remove IAM policy binding from an Apphub application."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddRemoveIamPolicyBindingFlags(parser)
    iam_util.AddArgsForRemoveIamPolicyBinding(parser)

  def Run(self, args):
    client = apis.ApplicationsClient(release_track=base.ReleaseTrack.GA)
    app_ref = api_lib_utils.GetApplicationRef(args)
    return client.RemoveIamPolicyBinding(
        app_id=app_ref.RelativeName(), member=args.member, role=args.role
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class RemoveIamPolicyBindingAlpha(base.Command):
  """Remove IAM policy binding from an Apphub application."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddRemoveIamPolicyBindingFlags(parser)
    iam_util.AddArgsForRemoveIamPolicyBinding(parser)

  def Run(self, args):
    client = apis.ApplicationsClient(release_track=base.ReleaseTrack.ALPHA)
    app_ref = api_lib_utils.GetApplicationRef(args)
    return client.RemoveIamPolicyBinding(
        app_id=app_ref.RelativeName(), member=args.member, role=args.role
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for checking and enabling necessary APIs."""

import urllib.parse as urlparse

from googlecloudsdk.api_lib.services import enable_api
from googlecloudsdk.api_lib.services import services_util
from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.api_lib.services.exceptions import GetServicePermissionDeniedException
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io

_RUN_API_NAMES = frozenset(
    ['run.googleapis.com', 'staging-run.sandbox.googleapis.com']
)


def get_run_api():
  endpoint = properties.VALUES.api_endpoint_overrides.run.Get()
  if endpoint:
    api = urlparse.urlparse(endpoint).hostname
    if api in _RUN_API_NAMES:
      return api
  return 'run.googleapis.com'


def check_and_enable_apis(project_id, required_apis):
  """Ensure the given APIs are enabled for the specified project."""
  if not properties.VALUES.core.should_prompt_to_enable_api.GetBool():
    # no need to even check if prompting is disabled.
    return False
  endpoint_mode = properties.VALUES.regional.endpoint_mode.Get()
  if (
      properties.VALUES.regional.endpoint_compatibility.Get()
      and endpoint_mode == properties.VALUES.regional.REGIONAL
  ):
    # Service usage API does not support regional endpoints.
    return False
  try:
    apis_not_enabled = get_disabled_apis(project_id, required_apis)
  except GetServicePermissionDeniedException:
    return False
  if apis_not_enabled:
    apis_to_enable = '\n\t'.join(apis_not_enabled)
    console_io.PromptContinue(
        default=True,
        cancel_on_no=True,
        message=(
            'The following APIs are not enabled on project [{0}]:\n\t{1}'
            .format(project_id, apis_to_enable)
        ),
        prompt_string='Do you want enable these APIs to '
        + 'continue (this will take a few minutes)?',
    )

    log.status.Print('Enabling APIs on project [{0}]...'.format(project_id))
    op = serviceusage.BatchEnableApiCall(project_id, apis_not_enabled)
    if not op.done:
      op = services_util.WaitOperation(op.name, serviceusage.GetOperation)
      services_util.PrintOperation(op)
  return True


def get_disabled_apis(project_id, required_apis):
  apis_not_enabled = [
      # iterable is sorted for scenario tests.  The order of API calls
      # should happen in the same order each time for the scenario tests.
      api
      for api in sorted(required_apis)
      if not enable_api.IsServiceEnabled(project_id, api)
  ]
  return apis_not_enabled



"""Generated client library for parametermanager version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.parametermanager.v1 import parametermanager_v1_messages as messages


class ParametermanagerV1(base_api.BaseApiClient):
  """Generated client library for service parametermanager version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://parametermanager.googleapis.com/'
  MTLS_BASE_URL = 'https://parametermanager.mtls.googleapis.com/'

  _PACKAGE = 'parametermanager'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ParametermanagerV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new parametermanager handle."""
    url = url or self.BASE_URL
    super(ParametermanagerV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_parameters_versions = self.ProjectsLocationsParametersVersionsService(self)
    self.projects_locations_parameters = self.ProjectsLocationsParametersService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsParametersVersionsService(base_api.BaseApiService):
    """Service class for the projects_locations_parameters_versions resource."""

    _NAME = 'projects_locations_parameters_versions'

    def __init__(self, client):
      super(ParametermanagerV1.ProjectsLocationsParametersVersionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new ParameterVersion in a given project, location, and parameter.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ParameterVersion) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions',
        http_method='POST',
        method_id='parametermanager.projects.locations.parameters.versions.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['parameterVersionId', 'requestId'],
        relative_path='v1/{+parent}/versions',
        request_field='parameterVersion',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsCreateRequest',
        response_type_name='ParameterVersion',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single ParameterVersion.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions/{versionsId}',
        http_method='DELETE',
        method_id='parametermanager.projects.locations.parameters.versions.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single ParameterVersion.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ParameterVersion) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions/{versionsId}',
        http_method='GET',
        method_id='parametermanager.projects.locations.parameters.versions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['view'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsGetRequest',
        response_type_name='ParameterVersion',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists ParameterVersions in a given project, location, and parameter.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListParameterVersionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions',
        http_method='GET',
        method_id='parametermanager.projects.locations.parameters.versions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/versions',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsListRequest',
        response_type_name='ListParameterVersionsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a single ParameterVersion.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ParameterVersion) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions/{versionsId}',
        http_method='PATCH',
        method_id='parametermanager.projects.locations.parameters.versions.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='parameterVersion',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsPatchRequest',
        response_type_name='ParameterVersion',
        supports_download=False,
    )

    def Render(self, request, global_params=None):
      r"""Gets rendered version of a ParameterVersion.

      Args:
        request: (ParametermanagerProjectsLocationsParametersVersionsRenderRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RenderParameterVersionResponse) The response message.
      """
      config = self.GetMethodConfig('Render')
      return self._RunMethod(
          config, request, global_params=global_params)

    Render.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}/versions/{versionsId}:render',
        http_method='GET',
        method_id='parametermanager.projects.locations.parameters.versions.render',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:render',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersVersionsRenderRequest',
        response_type_name='RenderParameterVersionResponse',
        supports_download=False,
    )

  class ProjectsLocationsParametersService(base_api.BaseApiService):
    """Service class for the projects_locations_parameters resource."""

    _NAME = 'projects_locations_parameters'

    def __init__(self, client):
      super(ParametermanagerV1.ProjectsLocationsParametersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new Parameter in a given project and location.

      Args:
        request: (ParametermanagerProjectsLocationsParametersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Parameter) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters',
        http_method='POST',
        method_id='parametermanager.projects.locations.parameters.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['parameterId', 'requestId'],
        relative_path='v1/{+parent}/parameters',
        request_field='parameter',
        request_type_name='ParametermanagerProjectsLocationsParametersCreateRequest',
        response_type_name='Parameter',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single Parameter.

      Args:
        request: (ParametermanagerProjectsLocationsParametersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}',
        http_method='DELETE',
        method_id='parametermanager.projects.locations.parameters.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Parameter.

      Args:
        request: (ParametermanagerProjectsLocationsParametersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Parameter) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}',
        http_method='GET',
        method_id='parametermanager.projects.locations.parameters.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersGetRequest',
        response_type_name='Parameter',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Parameters in a given project and location.

      Args:
        request: (ParametermanagerProjectsLocationsParametersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListParametersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters',
        http_method='GET',
        method_id='parametermanager.projects.locations.parameters.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/parameters',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsParametersListRequest',
        response_type_name='ListParametersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a single Parameter.

      Args:
        request: (ParametermanagerProjectsLocationsParametersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Parameter) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/parameters/{parametersId}',
        http_method='PATCH',
        method_id='parametermanager.projects.locations.parameters.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='parameter',
        request_type_name='ParametermanagerProjectsLocationsParametersPatchRequest',
        response_type_name='Parameter',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(ParametermanagerV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (ParametermanagerProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='parametermanager.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the [ListLocationsRequest.name] field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (ParametermanagerProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='parametermanager.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/locations',
        request_field='',
        request_type_name='ParametermanagerProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(ParametermanagerV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the parsing ouput for cloud build v2 API."""
from apitools.base.py import encoding
from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.api_lib.cloudbuild.v2 import output_util
from googlecloudsdk.core import yaml
from googlecloudsdk.core.resource import custom_printer_base


PRINTER_FORMAT = "tekton"


class TektonPrinter(custom_printer_base.CustomPrinterBase):
  """Print a  PipelineRun or TaskRun in Tekton YAML format."""

  def Transform(self, internal_proto):
    proto = encoding.MessageToDict(internal_proto)
    if (
        "pipelineSpec" in proto
        or "pipelineRef" in proto
        or "pipelineSpecYaml" in proto
    ):
      yaml_str = self.PublicPRToTektonPR(proto)
      return yaml.dump(yaml_str, round_trip=True)
    elif "taskSpec" in proto or "taskRef" in proto:
      yaml_str = self.PublicTRToTektonPR(proto)
      return yaml.dump(yaml_str, round_trip=True)

  def PublicPRToTektonPR(self, internal):
    """Convert a PipelineRun message into Tekton yaml."""
    pr = {
        "metadata": {},
        "spec": {},
        "status": {},
    }
    # METADATA
    if "name" in internal:
      pr["metadata"]["name"] = output_util.ParseName(
          internal.pop("name"), "pipelinerun"
      )
    if "annotations" in internal:
      pr["metadata"]["annotations"] = internal.pop("annotations")
    # SPEC
    if "params" in internal:
      pr["spec"]["params"] = _TransformParams(internal.pop("params"))
    if "pipelineSpec" in internal:
      pr["spec"]["pipelineSpec"] = _TransformPipelineSpec(
          internal.pop("pipelineSpec")
      )
    elif "pipelineRef" in internal:
      pr["spec"]["pipelineRef"] = TransformRef(internal.pop("pipelineRef"))
    elif "pipelineSpecYaml" in internal:
      yaml_string = internal.pop("pipelineSpecYaml")
      formatted_yaml = yaml.load(yaml_string, round_trip=True)
      pr["spec"]["pipelineSpecYaml"] = formatted_yaml
    if "timeout" in internal:
      pr["spec"]["timeout"] = internal.pop("timeout")
    if "workspaces" in internal:
      pr["spec"]["workspaces"] = internal.pop("workspaces")
    # STATUS
    if "conditions" in internal:
      conditions = internal.pop("conditions")
      pr["status"]["conditions"] = _TransformConditions(conditions)
    if "startTime" in internal:
      pr["status"]["startTime"] = internal.pop("startTime")
    if "completionTime" in internal:
      pr["status"]["completionTime"] = internal.pop("completionTime")
    # We set the resolvedPipelineSpec as status in Tekton
    if "resolvedPipelineSpec" in internal:
      rps = internal.pop("resolvedPipelineSpec")
      pr["status"]["pipelineSpec"] = _TransformPipelineSpec(rps)
    # PipelineRunResults
    if "results" in internal:
      pr["status"]["results"] = _TransformPipelineRunResults(
          internal.pop("results")
      )
    if "childReferences" in internal:
      crs = internal.pop("childReferences")
      pr["status"]["childReferences"] = crs
    # TASKRUNTEMPLATE
    if "serviceAccount" in internal:
      pr["taskRunTemplate"] = {
          "serviceAccountName": internal.pop("serviceAccount"),
      }
    return pr

  def PublicTRToTektonPR(self, internal):
    """Convert a TaskRun message into Tekton yaml."""
    tr = {
        "metadata": {},
        "spec": {},
        "status": {},
    }
    # METADATA
    if "name" in internal:
      tr["metadata"]["name"] = output_util.ParseName(
          internal.pop("name"), "taskrun"
      )
    # SPEC
    if "params" in internal:
      tr["spec"]["params"] = _TransformParams(internal.pop("params"))
    if "taskSpec" in internal:
      tr["spec"]["taskSpec"] = _TransformTaskSpec(internal.pop("taskSpec"))
    elif "taskRef" in internal:
      tr["spec"]["taskRef"] = TransformRef(internal.pop("taskRef"))
    if "timeout" in internal:
      tr["spec"]["timeout"] = internal.pop("timeout")
    if "workspaces" in internal:
      tr["spec"]["workspaces"] = internal.pop("workspaces")
    if "serviceAccountName" in internal:
      tr["spec"]["serviceAccountName"] = internal.pop("serviceAccountName")
    # STATUS
    if "conditions" in internal:
      tr["status"]["conditions"] = _TransformConditions(
          internal.pop("conditions")
      )
    if "startTime" in internal:
      tr["status"]["startTime"] = internal.pop("startTime")
    if "completionTime" in internal:
      tr["status"]["completionTime"] = internal.pop("completionTime")
    # We set the resolvedTaskSpec as the Status field in Tekton
    if "resolvedTaskSpec" in internal:
      rts = internal.pop("resolvedTaskSpec")
      tr["status"]["taskSpec"] = _TransformTaskSpec(rts)
    # StepState
    if "steps" in internal:
      tr["status"]["steps"] = _TransformStepStates(internal.pop("steps"))
    # TaskRunResults
    if "results" in internal:
      tr["status"]["results"] = _TransformTaskRunResults(
          internal.pop("results")
      )
    # SidecarState
    if "sidecars" in internal:
      tr["status"]["sidecars"] = internal.pop("sidecars")
    return tr


def _TransformPipelineSpec(ps):
  """Convert PipelineSpec into Tekton yaml."""
  pipeline_spec = {}
  if "params" in ps:
    pipeline_spec["params"] = TransformParamsSpec(ps.pop("params"))
  if "tasks" in ps:
    pipeline_spec["tasks"] = _TransformPipelineTasks(ps.pop("tasks"))
  if "results" in ps:
    pipeline_spec["results"] = _TransformPipelineResults(ps.pop("results"))
  if "finallyTasks" in ps:
    pipeline_spec["finally"] = _TransformPipelineTasks(ps.pop("finallyTasks"))
  if "workspaces" in ps:
    pipeline_spec["workspaces"] = ps.pop("workspaces")
  return pipeline_spec


def TransformParamsSpec(ps):
  """Convert ParamsSpecs into Tekton yaml."""
  param_spec = []
  for p in ps:
    param = {}
    if "name" in p:
      param["name"] = p.pop("name")
    if "description" in p:
      param["description"] = p.pop("description")
    if "type" in p:
      param["type"] = p.pop("type").lower()
    if "default" in p:
      param["default"] = _TransformParamValue(p.pop("default"))
    if "properties" in p:
      param["properties"] = p.pop("properties")
    param_spec.append(param)
  return param_spec


def _TransformTaskSpec(ts):
  """Convert TaskSpecs into Tekton yaml."""
  task_spec = {}
  if "params" in ts:
    task_spec["params"] = TransformParamsSpec(ts.pop("params"))
  if "steps" in ts:
    task_spec["steps"] = _TransformSteps(ts.pop("steps"))
  if "stepTemplate" in ts:
    task_spec["stepTemplate"] = ts.pop("stepTemplate")
  if "results" in ts:
    task_spec["results"] = _TransformTaskResults(ts.pop("results"))
  if "sidecars" in ts:
    task_spec["sidecars"] = ts.pop("sidecars")
  if "workspaces" in ts:
    task_spec["workspaces"] = ts.pop("workspaces")
  return task_spec


def _TransformOnError(oe):
  """Convert OnError into Tekton yaml."""
  return cloudbuild_util.SnakeToCamelString(oe.lower())


def _TransformSteps(steps):
  """Convert Steps into Tekton yaml."""
  results = []
  for step in steps:
    if "ref" in step:
      step["ref"] = TransformRef(step.pop("ref"))
    if "params" in step:
      step["params"] = _TransformParams(step.pop("params"))
    results.append(step)
    if "onError" in step:
      step["onError"] = _TransformOnError(step.pop("onError"))
  return results


def _TransformPipelineTasks(ts):
  """Convert PipelineTasks into Tekton yaml."""
  tasks = []
  for task in ts:
    t = {"name": task.get("name", None)}
    if "params" in task:
      t["params"] = _TransformParams(task.pop("params"))
    if "taskSpec" in task:
      task_spec = task.pop("taskSpec").pop("taskSpec")
      t["taskSpec"] = _TransformTaskSpec(task_spec)
    elif "taskRef" in task:
      t["taskRef"] = task.pop("taskRef")
    if "workspaces" in task:
      t["workspaces"] = task.pop("workspaces")
    if "runAfter" in task:
      t["runAfter"] = task.pop("runAfter")
    if "timeout" in task:
      t["timeout"] = task.pop("timeout")
    tasks.append(t)
  return tasks


def _TransformPipelineResults(rs):
  """Convert PipelineResults into Tekton yaml."""
  results = []
  for r in rs:
    result = {}
    if "name" in r:
      result["name"] = r.pop("name")
    if "description" in r:
      result["description"] = r.pop("description")
    if "type" in r:
      result["type"] = r.pop("type").lower()
    if "value" in r:
      result["value"] = _TransformResultValue(r.pop("value"))
    results.append(result)
  return results


def _TransformTaskResults(rs):
  """Convert TaskResults into Tekton yaml."""
  results = []
  for r in rs:
    result = {}
    if "name" in r:
      result["name"] = r.pop("name")
    if "description" in r:
      result["description"] = r.pop("description")
    if "type" in r:
      result["type"] = r.pop("type").lower()
    if "properties" in r:
      result["properties"] = r.pop("properties")
    if "value" in r:
      result["value"] = _TransformParamValue(r.pop("value"))
    results.append(result)
  return results


def _TransformPipelineRunResults(rs):
  """Convert PipelineRunResults into Tekton yaml."""
  results = []
  for r in rs:
    result = {}
    if "name" in r:
      result["name"] = r.pop("name")
    if "value" in r:
      result["value"] = _TransformResultValue(r.pop("value"))
    results.append(result)
  return results


def _TransformStepStates(steps):
  """Convert StepState into Tekton yaml."""
  step_states = []
  for s in steps:
    if "results" in s:
      s["results"] = _TransformTaskRunResults(s.pop("results"))
    step_states.append(s)
  return step_states


def _TransformTaskRunResults(rs):
  """Convert TaskRunResults into Tekton yaml."""
  results = []
  for r in rs:
    result = {}
    if "name" in r:
      result["name"] = r.pop("name")
    if "resultValue" in r:
      result["value"] = _TransformResultValue(r.pop("resultValue"))
    results.append(result)
  return results


def _TransformResultValue(v):
  """Convert ResultValue into Tekton yaml."""
  if "stringVal" in v:
    return v.pop("stringVal")
  if "arrayVal" in v:
    return v.pop("arrayVal")
  if "objectVal" in v:
    return v.pop("objectVal")
  return v


def _TransformParamValue(v):
  """Convert ParamValue into Tekton yaml."""
  if "stringVal" in v:
    return v.pop("stringVal")
  if "arrayVal" in v:
    return v.pop("arrayVal")
  return v


def _TransformParams(ps):
  """Convert Params into Tekton yaml."""
  params = []
  for p in ps:
    param = {}
    if "name" in p:
      param["name"] = p.pop("name")
    if "value" in p:
      param["value"] = _TransformParamValue(p.pop("value"))
    params.append(param)
  return params


def _TransformConditions(cs):
  """Convert Conditions into Tekton yaml."""
  conditions = []
  for c in cs:
    condition = {}
    # Only append the condition if it has a message
    # which indicates the final condition
    if "message" in c:
      condition["message"] = c.pop("message")
      if "lastTransitionTime" in c:
        condition["lastTransitionTime"] = c.pop("lastTransitionTime")
      if "status" in c:
        condition["status"] = c.pop("status").capitalize()
      if "type" in c:
        condition["type"] = c.pop("type").capitalize()
      if "reason" in c:
        condition["reason"] = c.pop("reason")
        conditions.append(condition)
  return conditions


def _TransformChildRefs(crs):
  """Convert ChildReferences into Tekton yaml."""
  child_refs = []
  for cr in crs:
    child_ref = {}
    if "name" in cr:
      child_ref["name"] = cr.pop("name")
    if "pipelineTask" in cr:
      child_ref["pipelineTask"] = cr.pop("pipelineTask")
    child_refs.append(child_ref)
  return child_refs


def TransformRef(ref):
  """Convert a generic reference (step, task, or pipeline) into Tekton yaml."""
  result = {}
  if "name" in ref:
    result["name"] = ref.pop("name")
  if "resolver" in ref:
    result["resolver"] = ref.pop("resolver")
  if "params" in ref:
    result["params"] = _TransformParams(ref.pop("params"))
  return result



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of CloudApi for s3 using boto3."""

# TODO(b/275749579): Rename this module.


import threading

import boto3
import botocore
from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import errors
from googlecloudsdk.api_lib.storage import headers_util
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.api_lib.storage import xml_metadata_field_converters
from googlecloudsdk.api_lib.storage import xml_metadata_util
from googlecloudsdk.command_lib.storage import errors as command_errors
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.command_lib.storage.resources import resource_util
from googlecloudsdk.command_lib.storage.resources import s3_resource_reference
from googlecloudsdk.command_lib.storage.tasks.cp import download_util
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import retry
from googlecloudsdk.core.util import scaled_integer
import s3transfer


# S3 does not allow upload of size > 5 GiB for put_object.
MAX_PUT_OBJECT_SIZE = 5 * (1024**3)  # 5 GiB
BOTO3_CLIENT_LOCK = threading.Lock()


def _raise_if_not_found_error(error, resource_name):
  if error.response.get('ResponseMetadata', {}).get('HTTPStatusCode') == 404:
    # TODO(b/193464904): Remove the hardcoded error message here after
    # refactoring the errors module.
    raise errors.NotFoundError('{} not found: 404.'.format(resource_name))


def _catch_client_error_raise_s3_api_error(format_str=None):
  """Decorator that catches botocore ClientErrors and raises XmlApiErrors.

  Args:
    format_str (str): A googlecloudsdk.api_lib.storage.errors.S3ErrorPayload
      format string. Note that any properties that are accessed here are on the
      S3ErrorPayload object, not the object returned from botocore.

  Returns:
    A decorator that catches botocore.exceptions.ClientError and returns an
      XmlApiError with a formatted error message.
  """

  return errors.catch_error_raise_cloud_api_error(
      [(botocore.exceptions.ClientError, None, errors.XmlApiError)],
      format_str=format_str,
  )


def _modifies_full_acl_policy(request_config):
  """Checks if RequestConfig has ACL setting aside from predefined ACL."""
  return bool(
      request_config.resource_args
      and (
          request_config.resource_args.acl_grants_to_add
          or request_config.resource_args.acl_grants_to_remove
          or request_config.resource_args.acl_file_path
      )
  )


# pylint:disable=abstract-method
class S3XmlClient(cloud_api.CloudApi):
  """S3 XML Client."""

  capabilities = {
      # Boto3 implements its own unskippable validation.
      cloud_api.Capability.CLIENT_SIDE_HASH_VALIDATION,
  }
  access_key_id = None
  access_key_secret = None
  scheme = storage_url.ProviderPrefix.S3

  def __init__(self):
    self.endpoint_url = properties.VALUES.storage.s3_endpoint_url.Get()
    self.client = self.create_client()

  def _add_additional_headers_to_request(self, request, **kwargs):
    del kwargs
    headers = headers_util.get_additional_header_dict()
    for key, value in headers.items():
      request.headers.add_header(key, value)

  def create_client(self, resource_location=None):
    """Creates the Boto3 client.

    Args:
      resource_location: (string) The name of the region associated with the
        client.

    Returns:
      A boto3 client.
    """
    disable_ssl_validation = (
        properties.VALUES.auth.disable_ssl_validation.GetBool())
    if disable_ssl_validation:
      verify_ssl = False
    else:
      # Setting to None so that AWS_CA_BUNDLE env variable
      #  can be used. See b/266098045.
      verify_ssl = None
    # Using a lock since the boto3.client creation is not thread-safe.
    with BOTO3_CLIENT_LOCK:
      client = boto3.client(
          storage_url.ProviderPrefix.S3.value,
          aws_access_key_id=self.access_key_id,
          aws_secret_access_key=self.access_key_secret,
          region_name=resource_location,
          endpoint_url=self.endpoint_url,
          verify=verify_ssl)
      # Adding headers to s3 calls requires registering an event handler.
      # https://github.com/boto/boto3/issues/2251
      client.meta.events.register_first(
          'before-sign.s3.*', self._add_additional_headers_to_request
      )
      return client

  @_catch_client_error_raise_s3_api_error()
  def create_bucket(self, bucket_resource, request_config, fields_scope=None):
    """See super class."""
    del fields_scope  # Unused in S3 client.

    resource_args = request_config.resource_args
    if resource_args.location:
      client = self.create_client(resource_args.location)
      location_constraint = resource_args.location
    else:
      client = self.client
      location_constraint = boto3.session.Session().region_name
    if location_constraint:
      metadata = client.create_bucket(
          Bucket=bucket_resource.storage_url.bucket_name,
          CreateBucketConfiguration={'LocationConstraint': location_constraint})
    else:
      metadata = client.create_bucket(
          Bucket=bucket_resource.storage_url.bucket_name)

    if (resource_args.cors_file_path or resource_args.labels_file_path or
        resource_args.lifecycle_file_path or resource_args.log_bucket or
        resource_args.log_object_prefix or resource_args.requester_pays or
        resource_args.versioning or resource_args.web_error_page or
        resource_args.web_main_page_suffix):
      return self.patch_bucket(bucket_resource, request_config)

    backend_location = metadata.get('Location')
    return s3_resource_reference.S3BucketResource(
        bucket_resource.storage_url,
        location=backend_location,
        metadata=metadata)

  @_catch_client_error_raise_s3_api_error()
  def delete_bucket(self, bucket_name, request_config):
    """See super class."""
    del request_config  # Unused.
    return self.client.delete_bucket(Bucket=bucket_name)

  def get_bucket(
      self,
      bucket_name,
      generation=None,
      fields_scope=cloud_api.FieldsScope.NO_ACL,
      soft_deleted=False,
  ):
    """See super class."""
    metadata = {'Name': bucket_name}
    # TODO(b/168716392) As new commands are implemented, they may want
    # specific error handling for different methods.
    try:
      # Low-bandwidth way to determine if bucket exists for FieldsScope.SHORT.
      metadata.update(self.client.get_bucket_location(
          Bucket=bucket_name))
    except botocore.exceptions.ClientError as error:
      _raise_if_not_found_error(error, bucket_name)

      metadata['LocationConstraint'] = errors.XmlApiError(error)

    if fields_scope is cloud_api.FieldsScope.SHORT:
      return xml_metadata_util.get_bucket_resource_from_xml_response(
          self.scheme, metadata, bucket_name
      )

    # Data for FieldsScope.NO_ACL.
    for key, api_call, result_has_key in [
        ('CORSRules', self.client.get_bucket_cors, True),
        ('ServerSideEncryptionConfiguration', self.client.get_bucket_encryption,
         True),
        ('LifecycleConfiguration',
         self.client.get_bucket_lifecycle_configuration, False),
        ('LoggingEnabled', self.client.get_bucket_logging, True),
        ('Payer', self.client.get_bucket_request_payment, True),
        ('Versioning', self.client.get_bucket_versioning, False),
        ('Website', self.client.get_bucket_website, False),
    ]:
      try:
        api_result = api_call(Bucket=bucket_name)
        # Some results are wrapped in dictionaries with keys matching "key".
        metadata[key] = api_result.get(key) if result_has_key else api_result
      except botocore.exceptions.ClientError as error:
        metadata[key] = errors.XmlApiError(error)

    # User requested ACL's with FieldsScope.FULL.
    if fields_scope is cloud_api.FieldsScope.FULL:
      try:
        metadata['ACL'] = self.client.get_bucket_acl(Bucket=bucket_name)
      except botocore.exceptions.ClientError as error:
        metadata['ACL'] = errors.XmlApiError(error)

    return xml_metadata_util.get_bucket_resource_from_xml_response(
        self.scheme, metadata, bucket_name
    )

  def list_buckets(self, fields_scope=cloud_api.FieldsScope.NO_ACL):
    """See super class."""
    try:
      response = self.client.list_buckets()
      for bucket in response['Buckets']:
        if fields_scope == cloud_api.FieldsScope.FULL:
          yield self.get_bucket(bucket['Name'], fields_scope)
        else:
          yield s3_resource_reference.S3BucketResource(
              storage_url.CloudUrl(storage_url.ProviderPrefix.S3,
                                   bucket['Name']),
              creation_time=resource_util.convert_datetime_object_to_utc(
                  bucket['CreationDate']),
              metadata={
                  'Bucket': bucket,
                  'Owner': response['Owner']
              })
    except botocore.exceptions.ClientError as error:
      core_exceptions.reraise(errors.XmlApiError(error))

  def _make_patch_request(self, bucket_resource, patch_function, patch_kwargs):
    patch_kwargs['Bucket'] = bucket_resource.storage_url.bucket_name
    try:
      patch_function(**patch_kwargs)
    except botocore.exceptions.ClientError as error:
      _raise_if_not_found_error(error, bucket_resource.storage_url.bucket_name)
      log.error(errors.XmlApiError(error))

  def patch_bucket(self,
                   bucket_resource,
                   request_config,
                   fields_scope=cloud_api.FieldsScope.NO_ACL):
    """See super class."""
    resource_args = request_config.resource_args
    if _modifies_full_acl_policy(request_config) or (
        request_config.predefined_acl_string
    ):
      put_acl_kwargs = {}
      if _modifies_full_acl_policy(request_config):
        if getattr(resource_args, 'acl_file_path', None):
          put_acl_kwargs['AccessControlPolicy'] = (
              xml_metadata_field_converters.process_acl_file(
                  resource_args.acl_file_path
              )
          )
        else:
          existing_acl_dict = self.client.get_bucket_acl(
              Bucket=bucket_resource.storage_url.bucket_name)
          put_acl_kwargs['AccessControlPolicy'] = (
              xml_metadata_util.get_acl_policy_with_added_and_removed_grants(
                  existing_acl_dict, request_config
              )
          )

      if request_config.predefined_acl_string:
        put_acl_kwargs['ACL'] = request_config.predefined_acl_string
      self._make_patch_request(bucket_resource, self.client.put_bucket_acl,
                               put_acl_kwargs)

    if resource_args.cors_file_path:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_cors,
          {
              'CORSConfiguration': xml_metadata_field_converters.process_cors(
                  resource_args.cors_file_path
              )
          },
      )

    if resource_args.labels_file_path:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_tagging,
          {
              'Tagging': xml_metadata_field_converters.process_labels(
                  resource_args.labels_file_path
              )
          },
      )

    if resource_args.lifecycle_file_path:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_lifecycle_configuration,
          {
              'LifecycleConfiguration': (
                  xml_metadata_field_converters.process_lifecycle(
                      resource_args.lifecycle_file_path
                  )
              ),
          },
      )

    # TODO(b/203088239): Fix patching so that all possible branches can be
    # tested.
    if resource_args.log_bucket or resource_args.log_object_prefix:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_logging,
          {
              'BucketLoggingStatus': (
                  xml_metadata_field_converters.process_logging(
                      resource_args.log_bucket, resource_args.log_object_prefix
                  )
              )
          },
      )

    if resource_args.requester_pays:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_request_payment,
          {
              'RequestPaymentConfiguration': (
                  xml_metadata_field_converters.process_requester_pays(
                      resource_args.requester_pays
                  )
              )
          },
      )

    if resource_args.versioning:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_versioning,
          {
              'VersioningConfiguration': (
                  xml_metadata_field_converters.process_versioning(
                      resource_args.versioning
                  )
              )
          },
      )

    if resource_args.web_error_page or resource_args.web_main_page_suffix:
      self._make_patch_request(
          bucket_resource,
          self.client.put_bucket_website,
          {
              'WebsiteConfiguration': (
                  xml_metadata_field_converters.process_website(
                      resource_args.web_error_page,
                      resource_args.web_main_page_suffix,
                  )
              )
          },
      )

    return self.get_bucket(
        bucket_resource.storage_url.bucket_name, fields_scope=fields_scope)

  @_catch_client_error_raise_s3_api_error()
  def copy_object(
      self,
      source_resource,
      destination_resource,
      request_config,
      posix_to_set=None,
      progress_callback=None,
      should_deep_copy_metadata=False,
      attributes_resource=None,
  ):
    """See super class."""
    if _modifies_full_acl_policy(request_config):
      acl_file_path = getattr(request_config.resource_args, 'acl_file_path',
                              None)
      if acl_file_path:
        acl_dict = xml_metadata_field_converters.process_acl_file(acl_file_path)
      else:
        existing_acl_dict = self.client.get_object_acl(
            Bucket=destination_resource.storage_url.bucket_name,
            Key=destination_resource.storage_url.resource_name)
        acl_dict = (
            xml_metadata_util.get_acl_policy_with_added_and_removed_grants(
                existing_acl_dict, request_config
            )
        )

      put_acl_kwargs = {
          'Bucket': destination_resource.storage_url.bucket_name,
          'Key': destination_resource.storage_url.resource_name,
          'AccessControlPolicy': acl_dict,
      }
      self.client.put_object_acl(**put_acl_kwargs)
    else:
      acl_dict = None

    source_kwargs = {'Bucket': source_resource.storage_url.bucket_name,
                     'Key': source_resource.storage_url.resource_name}
    if source_resource.storage_url.generation:
      source_kwargs['VersionId'] = source_resource.storage_url.generation

    copy_kwargs = {
        'Bucket': destination_resource.storage_url.bucket_name,
        'Key': destination_resource.storage_url.resource_name,
        'CopySource': source_kwargs,
    }

    if should_deep_copy_metadata:
      copy_kwargs['MetadataDirective'] = 'REPLACE'
      xml_metadata_util.copy_object_metadata(
          xml_metadata_util.copy_object_metadata(
              destination_resource.metadata,
              source_resource.metadata,
          ),
          copy_kwargs,
      )
    else:
      if xml_metadata_util.is_user_metadata_field_present_in_request_config(
          request_config,
          attributes_resource=attributes_resource,
          known_posix=posix_to_set,
      ):
        copy_kwargs['MetadataDirective'] = 'REPLACE'
        xml_metadata_util.copy_user_metadata_fields(
            source_resource.metadata, copy_kwargs
        )

    xml_metadata_util.update_object_metadata_dict_from_request_config(
        copy_kwargs, request_config, posix_to_set=posix_to_set
    )

    copy_response = self.client.copy_object(**copy_kwargs)
    if progress_callback:
      progress_callback(source_resource.size)
    return xml_metadata_util.get_object_resource_from_xml_response(
        self.scheme,
        copy_response,
        copy_kwargs['Bucket'],
        copy_kwargs['Key'],
        acl_dict=acl_dict,
    )

  @_catch_client_error_raise_s3_api_error()
  def delete_object(self, object_url, request_config):
    """See super class."""
    del request_config  # Unused.

    delete_object_kwargs = {
        'Bucket': object_url.bucket_name,
        'Key': object_url.resource_name,
    }
    if object_url.generation:
      delete_object_kwargs['VersionId'] = object_url.generation
    return self.client.delete_object(**delete_object_kwargs)

  def _download_object(self, cloud_resource, download_stream, digesters,
                       progress_callback, start_byte, end_byte):
    get_object_args = {
        'Bucket': cloud_resource.bucket,
        'Key': cloud_resource.name,
    }
    if end_byte is None:
      get_object_args['Range'] = 'bytes={}-'.format(start_byte)
    else:
      get_object_args['Range'] = 'bytes={}-{}'.format(start_byte, end_byte)

    if cloud_resource.generation is not None:
      get_object_args['VersionId'] = str(cloud_resource.generation)
    response = self.client.get_object(**get_object_args)
    processed_bytes = start_byte
    for chunk in response['Body'].iter_chunks(
        scaled_integer.ParseInteger(
            properties.VALUES.storage.download_chunk_size.Get())):
      download_stream.write(chunk)

      for hash_object in digesters.values():
        hash_object.update(chunk)

      processed_bytes += len(chunk)
      if progress_callback:
        progress_callback(processed_bytes)
    return response.get('ContentEncoding')

  def _download_object_resumable(self, cloud_resource, download_stream,
                                 digesters, progress_callback, start_byte,
                                 end_byte):
    progress_state = {'start_byte': start_byte, 'end_byte': end_byte}

    def _call_download_object():
      # We use this inner function instead of passing _download_object
      # directly because the Retryer function is not able to use the
      # updated args values.
      return self._download_object(
          cloud_resource, download_stream, digesters, progress_callback,
          progress_state['start_byte'], progress_state['end_byte'])

    def _should_retry_resumable_download(exc_type, exc_value, exc_traceback,
                                         state):
      for retryable_error_type in s3transfer.utils.S3_RETRYABLE_DOWNLOAD_ERRORS:
        if isinstance(exc_value, retryable_error_type):
          start_byte = download_stream.tell()
          if start_byte > progress_state['start_byte']:
            progress_state['start_byte'] = start_byte
            state.retrial = 0
          log.debug('Retrying download from byte {} after exception: {}.'
                    ' Trace: {}'.format(start_byte, exc_type, exc_traceback))
          return True
      return False

    retryer = retry.Retryer(
        max_retrials=properties.VALUES.storage.max_retries.GetInt(),
        wait_ceiling_ms=properties.VALUES.storage.max_retry_delay.GetInt() *
        1000,
        exponential_sleep_multiplier=(
            properties.VALUES.storage.exponential_sleep_multiplier.GetInt()))
    return retryer.RetryOnException(
        _call_download_object,
        sleep_ms=properties.VALUES.storage.base_retry_delay.GetInt() * 1000,
        should_retry_if=_should_retry_resumable_download)

  @_catch_client_error_raise_s3_api_error()
  def download_object(self,
                      cloud_resource,
                      download_stream,
                      request_config,
                      digesters=None,
                      do_not_decompress=False,
                      download_strategy=cloud_api.DownloadStrategy.ONE_SHOT,
                      progress_callback=None,
                      start_byte=0,
                      end_byte=None):
    """See super class."""
    del request_config, do_not_decompress  # Unused.
    if download_util.return_and_report_if_nothing_to_download(
        cloud_resource, progress_callback
    ):
      return None

    if digesters is not None:
      digesters_dict = digesters
    else:
      digesters_dict = {}

    if download_strategy == cloud_api.DownloadStrategy.RESUMABLE:
      content_encoding = self._download_object_resumable(
          cloud_resource, download_stream, digesters_dict, progress_callback,
          start_byte, end_byte)
    else:
      content_encoding = self._download_object(
          cloud_resource, download_stream, digesters_dict, progress_callback,
          start_byte, end_byte)

    return content_encoding

  @_catch_client_error_raise_s3_api_error()
  def get_object_metadata(
      self,
      bucket_name,
      object_name,
      request_config=None,
      generation=None,
      fields_scope=None,
      soft_deleted=False,
  ):
    """See super class."""
    del request_config, soft_deleted  # Unused.
    request = {'Bucket': bucket_name, 'Key': object_name}

    # The VersionId keyword argument to head_object is not nullable if it is
    # present, so only include it in the function call if it has a value.
    if generation is not None:
      request['VersionId'] = generation

    try:
      object_dict = self.client.head_object(**request)
    except botocore.exceptions.ClientError as e:
      _raise_if_not_found_error(
          e,
          storage_url.CloudUrl(storage_url.ProviderPrefix.S3, bucket_name,
                               object_name, generation).url_string)
      raise e

    # User requested ACL's with FieldsScope.FULL.
    if fields_scope is cloud_api.FieldsScope.FULL:
      try:
        acl_response = self.client.get_object_acl(**request)
        acl_response.pop('ResponseMetadata', None)
        object_dict['ACL'] = acl_response
      except botocore.exceptions.ClientError as error:
        object_dict['ACL'] = errors.XmlApiError(error)

    if fields_scope and fields_scope is not cloud_api.FieldsScope.SHORT:
      try:
        tags_response = self.client.get_object_tagging(**request)
        object_dict['TagSet'] = tags_response.get('TagSet', None)
      except Exception as error:  # pylint: disable=broad-except
        # TODO: b/436204166 - Remove this once the API supports object tags.
        # Currently GCS-XML API does not support object tags, and throws
        # an error when we try to fetch it. We will ignore the error for now and
        # continue. Users anyway do not interact with XML API from gcloud.
        log.warning('Failed to get object tags, error: %s', error)
        pass

    return xml_metadata_util.get_object_resource_from_xml_response(
        self.scheme, object_dict, bucket_name, object_name
    )

  def list_objects(
      self,
      bucket_name,
      prefix=None,
      delimiter=None,
      fields_scope=None,
      halt_on_empty_response=True,
      include_folders_as_prefixes=None,
      next_page_token=None,
      object_state=cloud_api.ObjectState.LIVE,
      list_filter=None,
  ):
    """See super class."""
    del (
        halt_on_empty_response,
        include_folders_as_prefixes,
        next_page_token,
        list_filter,
    )  # Only used by GCS.
    if object_state == cloud_api.ObjectState.LIVE_AND_NONCURRENT:
      api_method_name = 'list_object_versions'
      objects_key = 'Versions'
    else:
      api_method_name = 'list_objects_v2'
      objects_key = 'Contents'
    try:
      paginator = self.client.get_paginator(api_method_name)
      page_iterator = paginator.paginate(
          Bucket=bucket_name,
          Prefix=prefix if prefix is not None else '',
          Delimiter=delimiter if delimiter is not None else '')
      for page in page_iterator:
        for object_dict in page.get(objects_key, []):
          if fields_scope is cloud_api.FieldsScope.FULL:
            # The metadata present in the list_objects_v2 response or the
            # list_object_versions response is not enough
            # for a FULL scope. Hence, calling the GetObjectMetadata method
            # to get the additonal metadata and ACLs information.
            yield self.get_object_metadata(
                bucket_name=bucket_name,
                object_name=object_dict['Key'],
                request_config=request_config_factory.get_request_config(
                    storage_url.CloudUrl(scheme=storage_url.ProviderPrefix.S3)),
                generation=object_dict.get('VersionId'),
                fields_scope=fields_scope)
          else:
            yield xml_metadata_util.get_object_resource_from_xml_response(
                self.scheme, object_dict, bucket_name
            )
        for prefix_dict in page.get('CommonPrefixes', []):
          yield xml_metadata_util.get_prefix_resource_from_xml_response(
              self.scheme, prefix_dict, bucket_name
          )
    except botocore.exceptions.ClientError as error:
      core_exceptions.reraise(errors.XmlApiError(error))

  @_catch_client_error_raise_s3_api_error()
  def patch_object_metadata(
      self,
      bucket_name,
      object_name,
      object_resource,
      request_config,
      fields_scope=None,
      generation=None,
      posix_to_set=None,
  ):
    """See super class."""
    del fields_scope  # Unused.
    source_resource = self.get_object_metadata(
        bucket_name, object_name, generation=generation)
    return self.copy_object(
        source_resource=source_resource,
        destination_resource=object_resource,
        request_config=request_config,
        posix_to_set=posix_to_set,
        should_deep_copy_metadata=True,
    )

  def _upload_using_managed_transfer_utility(self, source_stream,
                                             destination_resource, extra_args):
    """Uploads the data using boto3's managed transfer utility.

    Calls the upload_fileobj method which performs multi-threaded multipart
    upload automatically. Performs slightly better than put_object API method.
    However, upload_fileobj cannot perform data intergrity checks and we have
    to use put_object method in such cases.

    Args:
      source_stream (a file-like object): A file-like object to upload. At a
        minimum, it must implement the read method, and must return bytes.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
        Represents the metadata for the destination object.
      extra_args (dict): Extra arguments that may be passed to the client
        operation.

    Returns:
      resource_reference.ObjectResource with uploaded object's metadata.
    """
    bucket_name = destination_resource.storage_url.bucket_name
    object_name = destination_resource.storage_url.resource_name
    multipart_chunksize = scaled_integer.ParseInteger(
        properties.VALUES.storage.multipart_chunksize.Get())
    multipart_threshold = scaled_integer.ParseInteger(
        properties.VALUES.storage.multipart_threshold.Get())
    self.client.upload_fileobj(
        Fileobj=source_stream,
        # TODO(b/276920544): Consider re-enabling parallelism pending resolution
        # from boto3 maintainers.
        Config=boto3.s3.transfer.TransferConfig(
            use_threads=False,
            multipart_chunksize=multipart_chunksize,
            multipart_threshold=multipart_threshold),
        Bucket=bucket_name,
        Key=object_name,
        ExtraArgs=extra_args)
    return self.get_object_metadata(
        bucket_name, object_name,
        request_config_factory.get_request_config(
            storage_url.CloudUrl(scheme=storage_url.ProviderPrefix.S3)))

  def _upload_using_put_object(self, source_stream, destination_resource,
                               extra_args):
    """Uploads the source stream using the put_object API method.

    Args:
      source_stream (a seekable file-like object): The stream of bytes to be
        uploaded.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
        Represents the metadata for the destination object.
      extra_args (dict): Extra arguments that may be passed to the client
        operation.

    Returns:
      resource_reference.ObjectResource with uploaded object's metadata.
    """
    kwargs = {
        'Bucket': destination_resource.storage_url.bucket_name,
        'Key': destination_resource.storage_url.resource_name,
        'Body': source_stream,
    }
    kwargs.update(extra_args)
    response = self.client.put_object(**kwargs)
    return xml_metadata_util.get_object_resource_from_xml_response(
        self.scheme,
        response,
        destination_resource.storage_url.bucket_name,
        destination_resource.storage_url.resource_name,
    )

  @_catch_client_error_raise_s3_api_error()
  def upload_object(
      self,
      source_stream,
      destination_resource,
      request_config,
      posix_to_set=None,
      serialization_data=None,
      source_resource=None,
      tracker_callback=None,
      upload_strategy=cloud_api.UploadStrategy.SIMPLE,
  ):
    """See super class."""
    del serialization_data, tracker_callback  # Unused.

    if upload_strategy == cloud_api.UploadStrategy.RESUMABLE:
      raise command_errors.Error(
          'Invalid upload strategy: {}.'.format(upload_strategy.value))

    # All fields common to both put_object and upload_fileobj are added
    # to the extra_args dict.
    extra_args = {}

    if isinstance(source_resource, resource_reference.ObjectResource):
      if source_resource.custom_fields:
        extra_args['Metadata'] = source_resource.custom_fields

    xml_metadata_util.update_object_metadata_dict_from_request_config(
        extra_args,
        request_config,
        attributes_resource=source_resource,
        posix_to_set=posix_to_set,
    )

    md5_hash = getattr(request_config.resource_args, 'md5_hash', None)
    if md5_hash:
      # The upload_fileobj method can perform multipart uploads, so it cannot
      # validate with user-provided MD5 hashes. Streaming uploads must use the
      # managed file transfer utility for retries in-flight.
      if upload_strategy is cloud_api.UploadStrategy.STREAMING:
        log.warning('S3 does not support MD5 validation for streaming uploads.')

      # Simple uploads can use the put_object API method if MD5 validation is
      # requested.
      elif upload_strategy is cloud_api.UploadStrategy.SIMPLE:
        if request_config.resource_args.size > MAX_PUT_OBJECT_SIZE:
          log.debug('The MD5 hash %s will be ignored', md5_hash)
          log.warning(
              'S3 does not support MD5 validation for the entire object if'
              ' size > %d bytes. File size: %d', MAX_PUT_OBJECT_SIZE,
              request_config.resource_args.size)
        else:
          if request_config.resource_args.size is not None:
            extra_args['ContentLength'] = request_config.resource_args.size
          return self._upload_using_put_object(source_stream,
                                               destination_resource, extra_args)

      # ContentMD5 might get populated for extra_args during request_config
      # translation. Remove it since upload_fileobj
      # does not accept ContentMD5.
      extra_args.pop('ContentMD5')

    # We default to calling the upload_fileobj method provided by boto3 which
    # is a managed-transfer utility that can perform multipart uploads
    # automatically. It can be used for non-seekable source_streams as well.
    return self._upload_using_managed_transfer_utility(
        source_stream, destination_resource, extra_args)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""app logs read command."""


from googlecloudsdk.api_lib.app import logs_util
from googlecloudsdk.api_lib.logging import common
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


class Read(base.Command):
  """Reads log entries for the current App Engine app."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.SERVICE.AddToParser(parser)
    flags.VERSION.AddToParser(parser)
    flags.LEVEL.AddToParser(parser)
    flags.LOGS.AddToParser(parser)
    parser.add_argument('--limit', required=False, type=int,
                        default=200, help='Number of log entries to show.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The list of log entries.
    """
    printer = logs_util.LogPrinter()
    printer.RegisterFormatter(logs_util.FormatRequestLogEntry)
    printer.RegisterFormatter(logs_util.FormatNginxLogEntry)
    printer.RegisterFormatter(logs_util.FormatAppEntry)
    project = properties.VALUES.core.project.Get(required=True)
    filters = logs_util.GetFilters(project, args.logs, args.service,
                                   args.version, args.level)

    lines = []
    # pylint: disable=g-builtin-op, For the .keys() method
    for entry in common.FetchLogs(log_filter=' AND '.join(filters),
                                  order_by='DESC',
                                  limit=args.limit):
      lines.append(printer.Format(entry))
    for line in reversed(lines):
      log.out.Print(line)

Read.detailed_help = {
    'DESCRIPTION': """\
        Display the latest log entries from stdout, stderr and crash log for the
        current Google App Engine app in a human readable format. This command
        requires that the caller have the logging.logEntries.list
        permission.
    """,
    'EXAMPLES': """\
        To display the latest entries for the current app, run:

          $ {command}

        To show only the entries with severity at `warning` or higher, run:

          $ {command} --level=warning

        To show only the entries with a specific version, run:

          $ {command} --version=v1

        To show only the 10 latest log entries for the default service, run:

          $ {command} --limit=10 --service=default

        To show only the logs from the request log for standard apps, run:

          $ {command} --logs=request_log

        To show only the logs from the request log for Flex apps, run:

          $ {command} --logs=nginx.request
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Zones service."""


from googlecloudsdk.api_lib.compute import constants


def List(compute_client, project):
  """Return list of zonal resources."""
  client = compute_client.apitools_client
  messages = compute_client.messages
  request = (client.zones,
             'List',
             messages.ComputeZonesListRequest(
                 project=project,
                 maxResults=constants.MAX_RESULTS_PER_PAGE))
  return compute_client.MakeRequests([request])



# Copyright 2026 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Builds an OrchestrationPipeline protobuf message from a dictionary."""

from typing import Any, Dict

from cloudsdk.google.protobuf import json_format
from orchestration_pipelines_models.pipeline_v1_model.protos.orchestration_pipeline_pb2 import (
    OrchestrationPipeline, )
from orchestration_pipelines_models.pipeline_v1_model.pipeline_validation import (
    PipelineValidator, )


class OrchestrationPipelineBuilder:
    """Builds an OrchestrationPipeline from a dictionary."""

    @classmethod
    def build(cls, pipeline_def: Dict[str, Any]) -> OrchestrationPipeline:
        """Maps a dictionary to an OrchestrationPipeline object."""
        pipeline = OrchestrationPipeline()
        json_format.ParseDict(js_dict=pipeline_def,
                              message=pipeline,
                              ignore_unknown_fields=False)
        PipelineValidator.validate(pipeline)
        return pipeline



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a new Assured Workloads environment."""


from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.base import ReleaseTrack
from googlecloudsdk.command_lib.assured import create_workload
from googlecloudsdk.command_lib.assured import flags


@base.ReleaseTracks(ReleaseTrack.GA)
class GaCreate(create_workload.CreateWorkload):
  """Create a new Assured Workloads environment."""

  @staticmethod
  def Args(parser):
    flags.AddCreateWorkloadFlags(parser, ReleaseTrack.GA)


@base.ReleaseTracks(ReleaseTrack.BETA)
class BetaCreate(create_workload.CreateWorkload):
  """Create a new Assured Workloads environment."""

  @staticmethod
  def Args(parser):
    flags.AddCreateWorkloadFlags(parser, ReleaseTrack.BETA)


@base.ReleaseTracks(ReleaseTrack.ALPHA)
class AlphaCreate(create_workload.CreateWorkload):
  """Create a new Assured Workloads environment."""

  @staticmethod
  def Args(parser):
    flags.AddCreateWorkloadFlags(parser, ReleaseTrack.ALPHA)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of command for describing managed folders."""


from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import errors_util
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage.resources import full_resource_formatter
from googlecloudsdk.command_lib.storage.resources import resource_util


@base.UniverseCompatible
class Describe(base.Command):
  """Describe managed folders."""

  hints = base.CommandHint(read_only=True)

  detailed_help = {
      'DESCRIPTION': """Describe managed folders.""",
      'EXAMPLES': """
      The following command shows information about a managed folder named
      `folder` in a bucket called `my-bucket`:

        $ {command} gs://my-bucket/folder/
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'url',
        type=str,
        help='The URL of the managed folder to describe.',
    )
    flags.add_additional_headers_flag(parser)
    flags.add_raw_display_flag(parser)

  def Run(self, args):
    url = storage_url.storage_url_from_string(args.url)
    errors_util.raise_error_if_not_gcs_folder_type(args.command_path, url)
    client = api_factory.get_api(url.scheme)
    resource = client.get_managed_folder(
        url.bucket_name,
        url.resource_name,
    )
    return resource_util.get_display_dict_for_resource(
        resource,
        full_resource_formatter.ManagedFolderDisplayTitlesAndDefaults,
        display_raw_keys=args.raw,
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementation of delete command for Insights dataset config."""

from googlecloudsdk.api_lib.storage import insights_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage.insights.dataset_configs import log_util
from googlecloudsdk.command_lib.storage.insights.dataset_configs import resource_args
from googlecloudsdk.core.console import console_io


@base.DefaultUniverseOnly
class Delete(base.Command):
  """Delete dataset config for Insights."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': """
      Delete an Insights dataset config.
      """,
      'EXAMPLES': """

      To delete a dataset config with config name "my_config" in location
      "us-central1":

          $ {command} my_config --location=us-central1

      To delete the same dataset config with fully specified name:

          ${command} projects/foo/locations/us-central1/datasetConfigs/my_config

      To delete the same dataset config and unlink it from the BigQuery
      instance:

          $ {command} my_config --location=us-central1 --auto-delete-link

      To delete the same dataset config without taking user consent:

          $ {command} my_config --location=us-central1 --auto-delete-link
          --force
      """,
  }

  @staticmethod
  def Args(parser):
    resource_args.add_dataset_config_resource_arg(parser, 'to delete')
    parser.add_argument(
        '--auto-delete-link',
        action='store_true',
        help=(
            'Delete the BigQuery instance links before the config gets deleted'
            ' explicitly.'
        ),
    )
    parser.add_argument(
        '--force',
        action='store_true',
        help='Force delete the config by skipping the consent.',
    )

  def Run(self, args):
    client = insights_api.InsightsApi()
    dataset_config_relative_name = (
        args.CONCEPTS.dataset_config.Parse().RelativeName()
    )

    if not args.force:
      message = 'You are about to delete dataset config: {}'.format(
          dataset_config_relative_name
      )
      console_io.PromptContinue(
          message=message, throw_if_unattended=True, cancel_on_no=True
      )

    if args.auto_delete_link:
      delete_dataset_config_link_operation = client.delete_dataset_config_link(
          dataset_config_relative_name,
      )
      log_util.dataset_config_operation_started_and_status_log(
          'Delete link',
          dataset_config_relative_name,
          delete_dataset_config_link_operation.name,
      )

    delete_dataset_config_operation = client.delete_dataset_config(
        dataset_config_relative_name
    )
    log_util.dataset_config_operation_started_and_status_log(
        'Delete',
        dataset_config_relative_name,
        delete_dataset_config_operation.name,
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to set an IAM policy binding on a Data Fusion instance."""


from googlecloudsdk.api_lib.data_fusion import datafusion as df
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.data_fusion import data_fusion_iam_util
from googlecloudsdk.command_lib.data_fusion import resource_args
from googlecloudsdk.command_lib.iam import iam_util


@base.RegionalEndpointsSupported
class AddIamPolicyBinding(base.Command):
  r"""Adds an IAM policy bindng to a Cloud Data Fusion instance.

  ## EXAMPLES

  To set someone@example.com to have `roles/datafusion.admin` permission for
  instance `my-instance`, in project `my-project`, location in `my-location`,
  run:

  $ {command} my-instance --location=my-location \
    --member=user:someone@example.com --role=roles/datafusion.admin

  To run the same command for a specific namespace on the instance, run:

  $ {command} my-instance --location=my-location \
    --member=user:someone@example.com --role=roles/datafusion.admin \
    --namespace=my-namespace
  """

  @staticmethod
  def Args(parser):
    resource_args.AddInstanceResourceArg(parser, 'Instance to add IAM binding.')
    base.URI_FLAG.RemoveFromParser(parser)

    iam_util.AddArgsForAddIamPolicyBinding(parser)
    parser.add_argument(
        '--namespace',
        help='CDAP Namespace whose IAM policy we wish to append. '
        'For example: `--namespace=my-namespace`.')

  def Run(self, args):
    instance_ref = args.CONCEPTS.instance.Parse()
    datafusion = df.Datafusion(location=instance_ref.locationsId)

    if not args.namespace:
      get_request = datafusion.messages.DatafusionProjectsLocationsInstancesGetIamPolicyRequest(
          resource=instance_ref.RelativeName())
      iam_policy = datafusion.client.projects_locations_instances.GetIamPolicy(
          get_request)
    else:
      get_request = datafusion.messages.DatafusionProjectsLocationsInstancesNamespacesGetIamPolicyRequest(
          resource='%s/namespaces/%s' %
          (instance_ref.RelativeName(), args.namespace))
      iam_policy = datafusion.client.projects_locations_instances_namespaces.GetIamPolicy(
          get_request)

    iam_util.AddBindingToIamPolicy(datafusion.messages.Binding,
                                   iam_policy,
                                   args.member,
                                   args.role)

    results = data_fusion_iam_util.DoSetIamPolicy(instance_ref, args.namespace,
                                                  iam_policy,
                                                  datafusion.messages,
                                                  datafusion.client)
    return results



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command group for Cloud API Gateway CLI."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ml_engine import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class ApiGateway(base.Group):
  """Manage Cloud API Gateway resources.

  Commands for managing Cloud API Gateway resources.
  """

  category = base.API_PLATFORM_AND_ECOSYSTEMS_CATEGORY

  def Filter(self, context, args):
    # TODO(b/190524392):  Determine if command group works with project number
    base.RequireProjectID(args)
    del context, args
    base.DisableUserProjectQuota()
    resources.REGISTRY.RegisterApiByName('apigateway', 'v1')



"""Generated client library for cloudasset version v1p5beta1."""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.cloudasset.v1p5beta1 import cloudasset_v1p5beta1_messages as messages


class CloudassetV1p5beta1(base_api.BaseApiClient):
  """Generated client library for service cloudasset version v1p5beta1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://cloudasset.googleapis.com/'
  MTLS_BASE_URL = 'https://cloudasset.mtls.googleapis.com/'

  _PACKAGE = 'cloudasset'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1p5beta1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'CloudassetV1p5beta1'
  _URL_VERSION = 'v1p5beta1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new cloudasset handle."""
    url = url or self.BASE_URL
    super(CloudassetV1p5beta1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.assets = self.AssetsService(self)

  class AssetsService(base_api.BaseApiService):
    """Service class for the assets resource."""

    _NAME = 'assets'

    def __init__(self, client):
      super(CloudassetV1p5beta1.AssetsService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Lists assets with time and resource types and returns paged results in.
response.

      Args:
        request: (CloudassetAssetsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAssetsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1p5beta1/{v1p5beta1Id}/{v1p5beta1Id1}/assets',
        http_method='GET',
        method_id='cloudasset.assets.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['assetTypes', 'contentType', 'pageSize', 'pageToken', 'readTime'],
        relative_path='v1p5beta1/{+parent}/assets',
        request_field='',
        request_type_name='CloudassetAssetsListRequest',
        response_type_name='ListAssetsResponse',
        supports_download=False,
    )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for the Access Context Manager levels CLI."""


from googlecloudsdk.calliope import base


class LevelCondition(base.Group):
  """Manage Access Context Manager level conditions.

  An access level is a classification of requests based on raw attributes of
  that request (e.g. IP address, device identity, time of day, etc.). These
  individual attributes are called conditions.
  """



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for flags in commands for Google Cloud Functions local development."""




def AddDeploymentNameFlag(parser):
  parser.add_argument(
      'NAME',
      nargs=1,
      help='Name of the locally deployed Google Cloud function.',
  )


def AddPortFlag(parser):
  parser.add_argument(
      '--port',
      default=8080,
      help='Port for the deployment to run on. The default port is 8080 '
      + 'for new local deployments.',
  )


def AddBuilderFlag(parser):
  parser.add_argument(
      '--builder',
      help=('Name of the builder to use for pack, e.g. '
            + '`gcr.io/serverless-runtimes/google-22-full/builder/go`.'),
  )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implements command to list the instance details for an OS patch job."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute.os_config import utils as osconfig_api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.os_config import resource_args
from googlecloudsdk.core.resource import resource_projector


def _TransformFailureReason(resource):
  max_len = 80  # Show the first 80 characters if failure reason is long.
  message = resource.get('failureReason', '')
  return (message[:max_len] + '..') if len(message) > max_len else message


def _PostProcessListResult(results):
  # Inject a "zone" field into the output resources for easy filtering.
  results_json = resource_projector.MakeSerializable(results)
  for result in results_json:
    result['zone'] = result['name'].split('/')[3]

  return results_json


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class ListInstanceDetails(base.ListCommand):
  """List the instance details for an OS patch job.

  ## EXAMPLES

  To list the instance details for each instance in the patch job `job1`, run:

        $ {command} job1

  """

  @staticmethod
  def Args(parser):
    base.URI_FLAG.RemoveFromParser(parser)  # InstanceDetails have no URI.
    resource_args.AddPatchJobResourceArg(parser, 'to list instance details.')
    parser.display_info.AddFormat("""
          table(
            name.basename(),
            zone,
            state,
            failure_reason()
          )
        """)
    parser.display_info.AddTransforms(
        {'failure_reason': _TransformFailureReason})

  def Run(self, args):
    patch_job_ref = args.CONCEPTS.patch_job.Parse()

    release_track = self.ReleaseTrack()
    client = osconfig_api_utils.GetClientInstance(release_track)
    messages = osconfig_api_utils.GetClientMessages(release_track)

    request = messages.OsconfigProjectsPatchJobsInstanceDetailsListRequest(
        pageSize=args.page_size, parent=patch_job_ref.RelativeName())

    results = list(
        list_pager.YieldFromList(
            client.projects_patchJobs_instanceDetails,
            request,
            limit=args.limit,
            batch_size=args.page_size,
            field='patchJobInstanceDetails',
            batch_size_attribute='pageSize'),)

    return _PostProcessListResult(results)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing organization firewall policies."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.firewall_policies import flags


@base.DefaultUniverseOnly
class List(base.ListCommand):
  """List Compute Engine organization firewall policies.

  *{command}* is used to list organization firewall policies. An organization
  firewall policy is a set of rules that controls access to various resources.
  """

  @classmethod
  def Args(cls, parser):
    flags.AddArgsListFirewallPolicy(parser)
    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
    lister.AddBaseListerArgs(parser)
    parser.display_info.AddCacheUpdater(flags.FirewallPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client.apitools_client
    messages = client.MESSAGES_MODULE
    if args.organization:
      parent_id = 'organizations/' + args.organization
    elif args.folder:
      parent_id = 'folders/' + args.folder

    request = messages.ComputeFirewallPoliciesListRequest(parentId=parent_id)
    return list_pager.YieldFromList(
        client.firewallPolicies,
        request,
        field='items',
        limit=args.limit,
        batch_size=None)


List.detailed_help = {
    'EXAMPLES':
        """\
    To list organization firewall policies under folder with ID
    ``123456789", run:

      $ {command} --folder=123456789
    """,
    'IAM PERMISSIONS': """\
    To list firewall policies, the user must have the following
    permission: *`compute.firewallPolicies.list`.

    To find predefined roles that contain those permissions, see the [Compute
    Engine IAM roles](https://cloud.google.com/compute/docs/access/iam).
    """
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""API lib for BigQuery."""

from googlecloudsdk.api_lib.util import apis


_BQ_API = 'bigquery'
_BQ_API_VERSION = 'v2'
_BQ_MIGRATION_API = 'bigquerymigration'
_BQ_MIGRATION_API_VERSION = 'v2'

TOOL_TAG_HEADER = 'tool-tag'
GCLOUD_TOOL_TAG = 'gcloud_cli'


def GetApiMessage(message_name, api=_BQ_API, api_version=_BQ_API_VERSION):
  """Return apitools message object for give message name."""
  messages = apis.GetMessagesModule(api, api_version)
  return getattr(messages, message_name)


def GetApiClient(api=_BQ_API, api_version=_BQ_API_VERSION):
  return apis.GetClientInstance(api, api_version)


def GetMigrationApiMessage(
    message_name, api=_BQ_MIGRATION_API, api_version=_BQ_MIGRATION_API_VERSION
):
  return GetApiMessage(message_name, api, api_version)


def GetMigrationApiClient():
  return GetApiClient(
      api=_BQ_MIGRATION_API, api_version=_BQ_MIGRATION_API_VERSION
  )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for Mirroring Deployment Group commands."""


from googlecloudsdk.api_lib.network_security.mirroring_deployment_groups import api
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs

DEPLOYMENT_GROUP_RESOURCE_NAME = "MIRRORING_DEPLOYMENT_GROUP"


def AddDeploymentGroupResource(release_track, parser):
  """Adds Mirroring Deployment Group resource."""
  resource_spec = concepts.ResourceSpec(
      "networksecurity.projects.locations.mirroringDeploymentGroups",
      "mirroring deployment group",
      api_version=api.GetApiVersion(release_track),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=concepts.ResourceParameterAttributeConfig(
          "location",
          "Location of the {resource}.",
          parameter_name="locationsId",
      ),
      mirroringDeploymentGroupsId=concepts.ResourceParameterAttributeConfig(
          "deployment-group-id",
          "Id of the {resource}",
          parameter_name="mirroringDeploymentGroupsId",
      ),
  )
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name=DEPLOYMENT_GROUP_RESOURCE_NAME,
      concept_spec=resource_spec,
      required=True,
      group_help="Mirroring Deployment Group.",
  )
  return concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddMaxWait(
    parser,
    default_max_wait,
    help_text="Time to synchronously wait for the operation to complete, after which the operation continues asynchronously. Ignored if --no-async isn't specified. See $ gcloud topic datetimes for information on time formats.",
):
  """Adds --max-wait flag."""
  parser.add_argument(
      "--max-wait",
      dest="max_wait",
      required=False,
      default=default_max_wait,
      help=help_text,
      type=arg_parsers.Duration(),
  )


def AddDescriptionArg(
    parser, help_text="Description of the mirroring deployment group"
):
  """Adds a resource argument for Google Cloud description."""
  parser.add_argument("--description", required=False, help=help_text)


def MakeGetUriFunc(release_track):
  return lambda x: api.GetEffectiveApiEndpoint(release_track) + x.name


def AddProjectArg(parser, help_text="Project ID of the deployment group"):
  parser.add_argument("--project", required=True, help=help_text)


def AddLocationArg(parser, help_text="Location of the deployment group"):
  parser.add_argument("--location", required=True, help=help_text)


def LocationAttributeConfig(default="global"):
  """Gets Google Cloud location resource attribute."""
  fallthroughs = []
  if default:
    fallthroughs.append(
        deps.Fallthrough(
            lambda: default,
            "Location of the Mirroring Deployment Group. Defaults to {}".format(
                default
            ),
        )
    )
  return concepts.ResourceParameterAttributeConfig(
      name="location",
      help_text="Location of the {resource}.",
      fallthroughs=fallthroughs,
  )


def GetLocationResourceSpec():
  """Constructs and returns the Resource specification for Location."""
  return concepts.ResourceSpec(
      "networksecurity.projects.locations",
      resource_name="location",
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddLocationResourceArg(
    parser: parser_arguments.ArgumentInterceptor, help_text
):
  """Adds a resource argument for Google Cloud location.

  Args:
    parser: The argparse.parser to add the resource arg to.
    help_text: str, the text of the help message.
  """
  concept_parsers.ConceptParser.ForResource(
      "--location",
      GetLocationResourceSpec(),
      help_text,
      required=True,
  ).AddToParser(parser)


def AddNetworkResource(parser):
  """Adds network resource."""
  resource_spec = concepts.ResourceSpec(
      "compute.networks",
      "network",
      api_version="v1",
      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      network=concepts.ResourceParameterAttributeConfig(
          "network-name",
          "Name of the {resource}",
          parameter_name="network",
      ),
  )
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name="--network",
      concept_spec=resource_spec,
      required=True,
      group_help="Mirroring Deployment Group.",
  )
  return concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Translation rule for networking features."""

from collections.abc import Mapping, Sequence
from typing import Any


def translate_network_features(
    input_data: Mapping[str, Any],
) -> Sequence[str]:
  """Translate networking features.

  Args:
    input_data: Flattened dictionary of the parsed app.yaml file.

  Returns:
    A list of strings representing the flags for Cloud Run.
  """
  output_flags = []
  if 'network.name' in input_data:
    output_flags.append(f'--network={input_data["network.name"]}')
  if 'network.subnetwork_name' in input_data:
    output_flags.append(f'--subnet={input_data["network.subnetwork_name"]}')
  if 'network.instance_tag' in input_data:
    output_flags.append(f'--network-tags={input_data["network.instance_tag"]}')
  if input_data.get('network.session_affinity'):
    output_flags.append('--session-affinity')

  forwarded_ports = input_data.get('network.forwarded_ports') or input_data.get(
      'network.forwardedPorts'
  )
  if forwarded_ports:
    output_flags.append(f'--port={forwarded_ports[0]}')

  instance_ip_mode = input_data.get(
      'network.instance_ip_mode'
  ) or input_data.get('network.instanceIpMode')
  if instance_ip_mode:
    if instance_ip_mode == 'external':
      output_flags.append('--ingress=all')
    elif instance_ip_mode == 'internal':
      output_flags.append('--ingress=internal')

  return output_flags



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the Datastream related commands."""



def AddDisplayNameFlag(parser):
  """Adds a --display-name flag to the given parser."""
  help_text = """Friendly name for the private connection."""
  parser.add_argument('--display-name', help=help_text, required=True)


def AddNetworkAttachmentFlag(parser):
  """Adds the `--network-attachment` flag to the parser."""
  parser.add_argument(
      '--network-attachment',
      required=True,
      type=str,
      help=(
          'Full URI of the network attachment that datastream will connect to.'
          'For example, this would be of the form:'
          '`network-attachment=projects/test-project/regions/us-central1/networkAttachments/my-na`'
      ),
  )


def AddValidateOnlyFlag(parser):
  """Adds the `--validate-only` flag to the parser."""
  parser.add_argument(
      '--validate-only',
      required=False,
      action='store_true',
      help=(
          'If set, the request will retrieve the project id to allow in the '
          ' network attachment Datastream will connect to.'
      ),
  )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to cancel a specified Batch job."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.batch import jobs
from googlecloudsdk.api_lib.batch import util
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.batch import resource_args
from googlecloudsdk.core import log


@base.UniverseCompatible
class Cancel(base.Command):
  """Cancel a job.

  This command can fail for the following reasons:
  * The job specified does not exist.
  * The active account does not have permission to cancel the given job.

  ## EXAMPLES

  To cancel the job with name
  `projects/foo/locations/us-central1/jobs/bar`, run:

    $ {command} projects/foo/locations/us-central1/jobs/bar
  """

  @staticmethod
  def Args(parser):
    resource_args.AddJobResourceArgs(parser)

  def Run(self, args):
    release_track = self.ReleaseTrack()

    client = jobs.JobsClient(release_track)
    job_ref = args.CONCEPTS.job.Parse()
    try:
      operation = client.Cancel(job_ref)
    except apitools_exceptions.HttpError as e:
      raise exceptions.HttpException(e, util.HTTP_ERROR_FORMAT)
    log.status.Print('Job {jobName} cancellation is in progress'.format(
        jobName=job_ref.RelativeName()))
    return operation



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Commands for creating or manipulating partner interconnect attachments."""


from googlecloudsdk.calliope import base


class InterconnectAttachments(base.Group):
  """Create or manipulate partner interconnect attachments."""
  pass

InterconnectAttachments.detailed_help = {
    'DESCRIPTION': """
        Create or manipulate Partner Interconnect attachments.

        For more information about interconnect attachments for Partner
        Interconnect, see the documentation for
        [Partner interconnect attachments](https://cloud.google.com/network-connectivity/docs/interconnect/how-to/dedicated/creating-vlan-attachments).

        See also: [Interconnect attachments API](https://cloud.google.com/compute/docs/reference/rest/v1/interconnectAttachments).
    """,
}
